#!/usr/bin/perl -w   
#
# texcal.pl: calculate and print a LaTeX calendar (on DIN A4 paper)
# Copyright (C) 1999  Manfred Maennle, Karlsruhe, Germany
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#
# Manfred Maennle
# Institut fuer Rechnerentwurf und Fehlertoleranz
# Universitaet Karlsruhe
# 76128 Karlsruhe
# Tel. 0721/608-6323, Fax 0721/608-3962
# maennle@computer.org
#
# $Id: texcal.pl,v 1.8 2000/11/30 17:53:18 Administrator Exp $
#
# usage:
# 1. define your special days and holidays in the respective section below
# 2. create LaTeX file, e.g.: texcal.pl <year> > file.tex
# 3. call LaTeX, e.g.: pdflatex file.tex
#


######################################################################
### you should not need to change these directives and variables

### compiler directives 
use strict;
use integer;

### modules
#use Time::Local;
#use File::Basename;

### variables
my %weekday;  # hash table using key "month:day"
my %holiday;  # hash table using key "month:day"
my %text;     # hash table using key "month:day"

my $null = "";
my $programname = "texcal.pl";
my $programversion = "\$Revision: 1.8 $null";
$programversion =~ s/(\$[a-zA-Z]+: )([\.0-9]+)( $null)/$2/g;

my $year;
my $y;
my $month;
my $day;
my $wday;
my $jan_1st;
my $eastermonth;
my $easterday;

my @months = ("1", "2", "3", "4", "5", "6", 
	      "7", "8", "9", "10", "11", "12"
	      );
my %monthlen = (
		"1" => 31, "2" => 28, "3" => 31,
		"4" => 30, "5" => 31, "6" => 30,
		"7" => 31, "8" => 31, "9" => 30,
		"10" => 31, "11" => 30, "12" => 31
		);


######################################################################
###   ********** DEFINE HOLIDAYS AND SPECIAL DAYS HERE **********
######################################################################

my %monthname = (
		"1" => "Januar", "2" => "Februar", "3" => "Mrz",
		"4" => "April", "5" => "Mai", "6" => "Juni",
		"7" => "Juli", "8" => "August", "9" => "September",
		"10" => "Oktober", "11" => "November", "12" => "Dezember"
		);
# my %dayname = (
# 	       "0" => "Montag", "1" => "Dienstag", "2" => "Mittwoch",
# 	       "3" => "Donnerstag", "4" => "Freitag", "5" => "Samstag",
# 	       "6" => "Sonntag"
# 	       );
my %dayname = (
	       "0" => "Mo", "1" => "Di", "2" => "Mi",
	       "3" => "Do", "4" => "Fr", "5" => "Sa",
	       "6" => "So"
	       );

### define fix holidays and special days here   <<<===================
### semantic: {"month:day"}; write "1" instead of "01"
sub special_days {

### your personal special days (your relatives' birthdays, etc.)
$text{"1:20"} = "birthday of texcal.pl";    


### holidays in Germany: 
#    1.1. Neujahr
#    6.1. Hl. Drei Knige
#    Karfreitag
#    Ostersonntag
#    Ostermontag
#    1.5. Maifeiertag
#    Christi Himmelfahrt
#    Pfingstsonntag
#    Pfingstmontag
#    Fronleichnam
#    15.8. Mari Himmelfahrt
#    3.10. Tag der dt. Einheit
#    1.11. Allerheiligen
#    Bu- und Bettag
#    25.12. Erster Weihnachtstag
#    26.12. Zweiter Weihnachtstag
#
### fix holidays 
$holiday{"1:1"} = 1; $text{"1:1"} = "Neujahr";
$holiday{"1:6"} = 1; $text{"1:6"} = "Hl. Drei Knige";
$text{"2:14"} = "Valentinstag";
$holiday{"5:1"} = 1; $text{"5:1"} = "Maifeiertag";
$holiday{"10:3"} = 1; $text{"10:3"} = "Tag der dt. Einheit";
$text{"3:21"} = "Frhlingsanfang";
$text{"6:21"} = "Sommeranfang";
$text{"8:15"} = "Mari Himmelfahrt";
$text{"9:23"} = "Herbstanfang";
$text{"10:31"} = "Reformationstag";
$holiday{"11:1"} = 1; $text{"11:1"} = "Allerheiligen";
$text{"11:11"} = "Martini";
$text{"12:22"} = "Winteranfang";
$text{"12:24"} = "Heiligabend";
$holiday{"12:25"} = 1; $text{"12:25"} = "1. Weihnachtstag";
$holiday{"12:26"} = 1; $text{"12:26"} = "2. Weihnachtstag";
$text{"12:31"} = "Silvester";

### holidays depending from Easter
($eastermonth, $easterday) = &easter($year);
if ($eastermonth eq "error") {
    ### no calculation of Easter possible
}
else {
    $holiday{"$eastermonth:$easterday"} = 1; 
    $text{"$eastermonth:$easterday"} = "Ostersonntag";

    ($month, $day) = &correct_date($eastermonth, $easterday - 52);
    $text{"$month:$day"} = "Schmutziger Donnerstag";
    ($month, $day) = &correct_date($eastermonth, $easterday - 48);
    $text{"$month:$day"} = "Rosenmontag";
    ($month, $day) = &correct_date($eastermonth, $easterday - 47);
    $text{"$month:$day"} = "Fasentszischdi";
    ($month, $day) = &correct_date($eastermonth, $easterday - 46);
    $text{"$month:$day"} = "Aschermittwoch";

    ($month, $day) = &correct_date($eastermonth, $easterday - 2);
    $holiday{"$month:$day"} = 1; 
    $text{"$month:$day"} = "Karfreitag";

    ($month, $day) = &correct_date($eastermonth, $easterday + 1);
    $holiday{"$month:$day"} = 1; 
    $text{"$month:$day"} = "Ostermontag";

    ($month, $day) = &correct_date($eastermonth, $easterday + 7);
    $text{"$month:$day"} = "Weier Sonntag";

    ($month, $day) = &correct_date($eastermonth, $easterday + 39);
    $holiday{"$month:$day"} = 1; 
    $text{"$month:$day"} = "Christi Himmelfahrt";

    ($month, $day) = &correct_date($eastermonth, $easterday + 49);
    $holiday{"$month:$day"} = 1; 
    $text{"$month:$day"} = "Pfingstsonntag";

    ($month, $day) = &correct_date($eastermonth, $easterday + 50);
    $holiday{"$month:$day"} = 1; 
    $text{"$month:$day"} = "Pfingstmontag";

    ($month, $day) = &correct_date($eastermonth, $easterday + 60);
    $holiday{"$month:$day"} = 1; 
    $text{"$month:$day"} = "Fronleichnam";
    
}


### holidays in France: 
#    1.1. Jour de l'An 
#    Pques
#    Lundi de Pques
#    1.5. Fte de Travail   
#    8.5. Victoire 1945
#    Ascension
#    Pentecte
#    Lundi de Pentecte
#    14.7. Fte Nationale
#    15.8. Assomption
#    1.11. Toussaint
#    11.11. Victoire 1918
#    25.12. Nol

return;
}


######################################################################
### ********** YOU SHOULD NOT NEED TO CHANGE ANYTHING BELOW **********
######################################################################


sub helpmessage {
    print "$programname (Version $programversion):";
    print " print a calendar in LaTeX format (for DIN A4 paper)\n";
    print "usage: $programname <year>\n"
}


sub leapyear {
    ### return 1 if year is a leap year, else 0 
    my $year = $_[0];
    if ( (($year%4 == 0) && ($year%100 != 0)) || ($year%400 == 0) ) {
	return 1;
    }
    return 0;
}

sub easter {
    ### return month ("1".."12") and day (1..31) of Easter sunday
    ### the algorithm is from Carl Friedrich Gauss (1777-1855)
    ### source: http://www.uni-bamberg.de/~ba1lw1/fkal.html
    my $year = $_[0];
    my ($m, $n);
    my ($a, $b, $c, $d, $e);
    if ( ($year>=1700) && ($year<1800) ) {
	$m = 23;
	$n = 3;
    }
    elsif ( ($year>=1800) && ($year<1900) ) {
	$m = 23;
	$n = 4;
    }
    elsif ( ($year>=1900) && ($year<2100) ) {
	$m = 24;
	$n = 5;
    }
    elsif ( ($year>=2100) && ($year<2200) ) {
	$m = 24;
	$n = 6;
    }
    else {
	return "error", 0;
    }
    $a = $year % 19;
    $b = $year % 4;
    $c = $year % 7;
    $d = (19 * $a + $m) % 30;
    $e = (2 * $b + 4 * $c + 6 * $d + $n) % 7;
    $month = 3;
    $day = 22 + $d + $e;
    if ($day > 31) {
	$day -= 31;
	$month = 4;
	if ($day == 26) {
	    $day = 19;
	}
	if ( ($day == 25) && ($d == 28) && ($a > 10) ) {
	    $day = 18;
	}
    }
    return $month, $day;
}

sub correct_date {
    ### correct a given date (within one year) 
    my $month = $_[0];
    my $day = $_[1];
    while ($day < 1) {
	if ($month < 2) {
	    print "$programname: error in `correct_date': underflow\n";
	    exit -1;
	}
	--$month;
	$day += $monthlen{$month};
    }
    while ($day > $monthlen{$month}) {
	if ($month > 11) {
	    print "$programname: error in `correct_date': overflow\n";
	    exit -1;
	}
	$day -= $monthlen{$month};
	++$month;
    }
    return $month, $day;
}


######################################################################

### scan command line
if (@ARGV < 1) {
    print "\n$programname: error in command line\n\n";
    &helpmessage();
    exit 0;
}
if ($ARGV[0] eq "-h") {
    &helpmessage();
    exit 0;
}
$year = $ARGV[0];
### check command line parameters
if (! defined($year) or $year < 1590) {
    print "$programname: error: cannot calculate calendar for years"
	. " before 1590\n";
    print "Type `$programname -h' for help!\n";
    exit(-1);
}

### correct number of days in february in case of a leap year
if (&leapyear($year)) {
    $monthlen{"2"} = 29;
}

### calculate the weekday of jan 1st; january 1st of 1590 was a monday 
$jan_1st = 0;
for ($y=1590; $y<$year; $y++) { 
    ### calculate weekday of the next year 
    if (&leapyear($y)) {
	$jan_1st = ($jan_1st + 2) % 7;
    }
    else {
	$jan_1st = ($jan_1st + 1) % 7;
    }
}

### calculate weekdays
$wday = $jan_1st;
foreach $month (@months) {
    for ($day=1; $day<=$monthlen{$month}; ++$day) {
	$weekday{"$month:$day"} = $dayname{$wday};
	$wday = ($wday + 1) % 7;
	if ($wday == 0 or $wday == 6) {
	    $holiday{"$month:$day"} = 1;
	}
	else {
	    $holiday{"$month:$day"} = 0;
	}
    }
}

### include holidays and special days
&special_days();


# foreach $month (@months) {
#     for ($day=1; $day<=$monthlen{$month}; ++$day) {
# 	my $key = "$month:$day";
# 	if (exists $text{$key}) {
# 	    print "$monthname{$month} $day ($weekday{$key}): $text{$key}\n";
# 	}
#     }
# }
# exit 0;

### print LaTeX header
print "% calendar for the year $year\n";
print "% creator: $programname v$programversion\n\n";
print "\\documentclass[10pt,a4paper]{report}\n\n";
print "\\usepackage[T1]{fontenc}\n";
print "\\usepackage[latin1]{inputenc}\n";
print "\\usepackage{times}\n\n";

print "\\pagestyle{empty}\n";
print "\\topmargin -3.3cm\n";
print "\\textwidth 20cm\n";
print "\\oddsidemargin -2.04cm\n";
print "\\evensidemargin -2.04cm\n";
print "\\textheight 28.44cm\n\n";

print "\\begin{document}\n";
print "\\scriptsize\n";

print "\\centerline{\\small\\textsc{Kalender fr das Jahr $year}}";
print "\\smallskip\n";

### print LaTeX calendar (two tabulars)
my $halfyear;
for($halfyear = 0; $halfyear < 2; $halfyear++) {
    if ($halfyear == 1) {
	print "\\newpage\n\n";
    }
    print "\\begin{tabular}{|l|l|l|l|}\n";
    print "\\hline\n";
    my $first_month_left;
    my $first_month_right;
    if ($halfyear == 0) { 
	$first_month_left = 1;
	$first_month_right = 4;
    }
    else { 
	$first_month_left =  7;
	$first_month_right = 10;
    }
    my $month_left = $first_month_left;
    my $month_right = $first_month_right;
    my $day_left = 0;
    my $day_right = 0;
    while ( ($month_left < $first_month_left + 3) 
	   || ($month_right < $first_month_right + 3) 
	   ) {
	### print element of left column
	if ($month_left < $first_month_left + 3) {
	    ### print left entry
	    if ($day_left == 0) {
		### print monthname
		print "\\multicolumn{2}{|l|}{\\textbf{";
		print "$monthname{$month_left} $year}} & ";
		++$day_left;
	    }
	    else {
		### print dayname
		my $key = "$month_left:$day_left";
		if ($holiday{$key}) {
		    print "\\textbf{";
		}
		if ($day_left < 10) {
		    print "0";
		}
		print "$day_left $weekday{$key}";
		if ($holiday{$key}) {
		    print "} & ";
		}
		else {
		    print " & ";
		}
		if (exists $text{$key}) {
		    print $text{$key};
		}
		else {
		    print "\\rule{8cm}{0pt}";
		}
		print " & ";
		++$day_left;
		if ($day_left > $monthlen{$month_left}) { 
		    $day_left = 0;
		    ++$month_left;
		}
		
	    }
	} ### end of print left entry
	else {
	    ### empty entry in left column needed 
	    if ($month_right < $first_month_right + 3) {
		print " & \\rule{8cm}{0pt} & "; 
	    }
	}
	### print element of right column
	if ($month_right < $first_month_right + 3) {
	    ### print right entry
	    if ($day_right == 0) {
		### print monthname
		print "\\multicolumn{2}{|l|}{\\textbf{";
		print "$monthname{$month_right} $year}}";
		print "\\\\\\hline\n";
		++$day_right;
	    }
	    else {
		### print dayname
		my $key = "$month_right:$day_right";
		if ($holiday{$key}) {
		    print "\\textbf{";
		}
		if ($day_right < 10) {
		    print "0";
		}
		print "$day_right $weekday{$key}";
		if ($holiday{$key}) {
		    print "} & ";
		}
		else {
		    print " & ";
		}
		if (exists $text{$key}) {
		    print $text{$key};
		}
		else {
		    print "\\rule{8cm}{0pt}";
		}
		print "\\\\\\hline\n";
		++$day_right;
		if ($day_right > $monthlen{$month_right}) { 
		    $day_right = 0;
		    ++$month_right;
		}
	    } ### end of print dayname
	} ### end of print right entry
	else {
	    ### empty entry in right column needed 
	    if ($month_left < $first_month_left + 3) {
		print " & \\rule{8cm}{0pt}"; 
		print "\\\\\\hline\n";
	    }
	}
    } ### end of while ...
    ### both columns printed; end of tabular
    print "\\end{tabular}\n";
}

### ok, that's it
print "\\end{document}\n";
exit 0;
